/*==============================================================================
		オートコンプリート付きテキストコントロール	for ZETA	Ver 1.41
		Copyright (C) 2004-2006 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2004/09/17 (Fri) 開発着手。
 1.00 | 2004/09/19 (Sun) オリジナルバージョン。
 1.10 | 2004/09/24 (Fri) 候補の選択によって順位が変わるようにした。
 1.20 | 2004/09/25 (Sat) 候補を自動学習（確定時学習）するようにした。
 1.30 | 2004/09/25 (Sat) SetMaxCandidates() を作った。
 1.40 | 2004/09/25 (Sat) SetTo() を作った。
==============================================================================*/

/*=== Memo =====================================================================
SetMaxCandidates()
SetTo()
Instances と候補の共有

LView を自作しないとダメだ
	-> シングルクリックで選択できるように
	-> BS で候補ウィンドウを消せるように
終了時に落ちることがある。フィルタを delete しているからか？
==============================================================================*/

/*=== 学習 =====================================================================
1. プログラム的学習：AddCandidate()/RemoveCandidate() による明示的な候補の追加と削除
2. 候補選択：Shift+Enter や候補ウィンドウでの選択による候補順序の入れ替え
3. 確定時学習：Enter や Exit などに候補を自動抽出して追加
	トップレベルターゲット（最後のトリガー以降）が候補に無い：候補の先頭に追加
	候補にある：候補の 7 番目（候補ウィンドウの下から 3 番目）まで浮上
		セカンドレベルターゲットが候補に無い：候補の 7 番目（先頭ではない）に追加
		候補にある：候補の 7 番目（候補ウィンドウの下から 3 番目）まで浮上
			サードレベル以降も同様
==============================================================================*/

//------------------------------------------------------------------------------
#include "ACTextControl.h"
//------------------------------------------------------------------------------
// BeOS
#include <Autolock.h>
#include <File.h>
#include <Locale.h>
#include <ScrollBar.h>
#include <ScrollView.h>
#include <interface/StringView.h>
// C++
#include <algorithm>
// Add2
#include <add2/FUConstants.h>
#include <add2/MiscDebug.h>
#include <add2/MiscUtils.h>
//------------------------------------------------------------------------------
// コマンド
const uint32	CANDIDATE_DETERED = 'ACcd';
const uint32	CANDIDATE_WINDOW_CLOSED = 'ACcw';
const uint32	LVIEW_CANDIDATES_SELECTED = 'AClc';
const uint32	MATE_CANDIDATES_UPDATED = 'ACmc';
const uint32	TBUTTON_SORT_CLICKED = 'ACst';
//------------------------------------------------------------------------------
const char		ARC_NAME_CANDIDATE[] = "_Cdd";
const char		MSG_NAME_CANDIDATE[] = "Candidate";
const char		MSG_NAME_STORAGE_FILE_NAME[] = "StorageFName";
const int32		INSERT_CANDIDATE_POSITION = 6;	// 0 ベース
const uint32	DEFAULT_MAX_CANDIDATES = 18;
const float		HEIGHT_SCRL_VIEW_OCCUPY = 2.0;
const float		MIN_CANDIDATES_VIEW_WIDTH = 100.0;
const float		WIDTH_SCRL_VIEW_OCCUPY = 8.0;
//------------------------------------------------------------------------------
BLocker	TACTextControl::smInstancesLocker;
list<TACTextControl*>	TACTextControl::smInstances;
//==============================================================================
TACCandidatesView::TACCandidatesView(BRect oFrame, BHandler* oTgt)
		: SUPER(oFrame, "ACCandidatesView")
{
	BScrollView*	aScrlView;

	// メンバー設定
	mTgt = oTgt;
	// ビュー作成
		// リスト
	mLViewCandidates = new BListView(BRect(0, 0, MIN_CANDIDATES_VIEW_WIDTH-B_V_SCROLL_BAR_WIDTH, 1), "LViewCandidates",
			B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES);
	aScrlView = new BScrollView("ScrlView", mLViewCandidates, B_FOLLOW_ALL_SIDES, 0, false, true);
	AddChild(aScrlView);
		// ツールバー
	mToolBar = new TToolBar(BRect(0, aScrlView->Frame().bottom+1, MIN_CANDIDATES_VIEW_WIDTH, 1), "ToolBar");
	mToolBar->SetResizingMode(B_FOLLOW_LEFT_RIGHT|B_FOLLOW_BOTTOM);
	AddChild(mToolBar);
		// ボタン
	mTButtonLRU = new TToolButton(BRect(0, 0, 1, 1), "TBLRU", NULL, "", new BMessage(TBUTTON_SORT_CLICKED));
	mTButtonLRU->SetBehavior(B_TWO_STATE_BUTTON);
	mToolBar->AddItem(mTButtonLRU);
	mTButtonAlpha = new TToolButton(BRect(0, 0, 1, 1), "TBAlpha", NULL, "", new BMessage(TBUTTON_SORT_CLICKED));
	mTButtonAlpha->SetBehavior(B_TWO_STATE_BUTTON);
	mToolBar->AddItem(mTButtonAlpha);
	// ビュー設定
	mLViewCandidates->SetInvocationMessage(new BMessage(LVIEW_CANDIDATES_SELECTED));
	mTButtonLRU->SetRadioMode(true);
	mTButtonLRU->SetValue(B_CONTROL_ON);
	ResizeTo(MIN_CANDIDATES_VIEW_WIDTH, mToolBar->Frame().bottom);
	SetResizingMode(B_FOLLOW_ALL_SIDES);
}
//------------------------------------------------------------------------------
TACCandidatesView::~TACCandidatesView()
{
}
//------------------------------------------------------------------------------
void	TACCandidatesView::AllAttached()
{
	SetTargetToThis(this);
	mLViewCandidates->SetTarget(this);
	mLViewCandidates->MakeFocus(true);
	SetViewLabels();
}
//------------------------------------------------------------------------------
void	TACCandidatesView::Cancel()
{
	BMessage	aSendMsg(CANDIDATE_WINDOW_CLOSED);

	BMessenger(mTgt).SendMessage(&aSendMsg);
	Window()->Quit();
}
//------------------------------------------------------------------------------
void	TACCandidatesView::Deter()
{
	list<string>::iterator	p;
	BMessage	aSendMsg(CANDIDATE_DETERED);

	if ( mLViewCandidates->CurrentSelection() < 0 )
		return;
	p = mDisplayCandidates.begin();
	for ( int32 i = 0 ; i < mLViewCandidates->CurrentSelection() ; i++ ) {
		p++;
	}
	aSendMsg.AddString(MSG_NAME_CANDIDATE, p->c_str());
	BMessenger(mTgt).SendMessage(&aSendMsg);
	Cancel();
}
//------------------------------------------------------------------------------
void	TACCandidatesView::LViewCandidatesSelected()
{
	Deter();
}
//------------------------------------------------------------------------------
void	TACCandidatesView::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case LVIEW_CANDIDATES_SELECTED:
		LViewCandidatesSelected();
		break;
	case TBUTTON_SORT_CLICKED:
		TButtonSortClicked();
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
void	TACCandidatesView::SetCandidates(const list<string>& oCandidates)
{
	int32			aNumDisplayItems;
	float			aWidth = MIN_CANDIDATES_VIEW_WIDTH;
	list<string>::iterator	p;
	BAutolock		aLock(static_cast<BWindow*>(Window()));
	BStringItem*	aSItem;

//	DBEXP("TACCandidatesView::SetCandidates - mLViewCandidates", mLViewCandidates->Frame());
//	DBEXP("TACCandidatesView::SetCandidates - mToolBar", mToolBar->Frame());
	// 旧アイテムを削除
	mLViewCandidates->DeselectAll();
	for ( int32 i = mLViewCandidates->CountItems()-1 ; i >= 0 ; i-- ) {
		delete mLViewCandidates->RemoveItem(i);
	}
	// 新アイテム登録
	mDisplayCandidates = mCandidates = oCandidates;
	if ( mTButtonAlpha->Value() != B_CONTROL_OFF ) {
		mDisplayCandidates.sort();
	}
	p = mDisplayCandidates.begin();
	while ( p != mDisplayCandidates.end() ) {
		aSItem = new BStringItem(p->c_str());
		mLViewCandidates->AddItem(aSItem);
		aWidth = max_c(aWidth, mLViewCandidates->StringWidth(p->c_str()));
		p++;
	}
	// リサイズ
	aNumDisplayItems = min_c(AC_TXT_CTRL_DISPLAY_MAX_CANDIDATES, mDisplayCandidates.size());
	Window()->ResizeTo(aWidth+B_V_SCROLL_BAR_WIDTH+WIDTH_SCRL_VIEW_OCCUPY,
			(mLViewCandidates->ItemAt(0)->Height()+1)*aNumDisplayItems+HEIGHT_SCRL_VIEW_OCCUPY+mToolBar->Bounds().bottom);
	// 選択
//	mLViewCandidates->SetTarget(NULL);
	mLViewCandidates->Select(0);
//	mLViewCandidates->SetTarget(this);
}
//------------------------------------------------------------------------------
void	TACCandidatesView::SetViewLabels()
{
	mTButtonLRU->SetLabel(_T(LKK_ACC_VIEW_TBUTTON_SORT_LRU));
	mTButtonAlpha->SetLabel(_T(LKK_ACC_VIEW_TBUTTON_SORT_ALPHA));
	mToolBar->RelocateItems();
}
//------------------------------------------------------------------------------
void	TACCandidatesView::ShortCutDown(int32 oChar, int32 oFunc)
{
	switch ( oChar ) {
	case B_ESCAPE:
		Cancel();
		break;
	case B_ENTER:
		Deter();
		break;
	default:
		SUPER::ShortCutDown(oChar, oFunc);
	}
}
//------------------------------------------------------------------------------
void	TACCandidatesView::TButtonSortClicked()
{
	SetCandidates(mCandidates);
}
//------------------------------------------------------------------------------
//==============================================================================
TACCandidatesWindow::TACCandidatesWindow(BRect oFrame, BHandler* oTgt)
		: SUPER(oFrame, "ACCandidatesWindow", B_FLOATING_WINDOW, B_ASYNCHRONOUS_CONTROLS|B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
{
	mCandidatesView = new TACCandidatesView(BRect(0, 0, 1, 1), oTgt);
	ResizeTo(mCandidatesView->Bounds().right, mCandidatesView->Bounds().bottom);
//	DBEXP("TACCandidatesWindow::TACCandidatesWindow", Bounds());
	AddChild(mCandidatesView);
}
//------------------------------------------------------------------------------
TACCandidatesWindow::~TACCandidatesWindow()
{
}
//------------------------------------------------------------------------------
bool	TACCandidatesWindow::QuitRequested()
{
	BMessage*	aCurMsg;

	aCurMsg = CurrentMessage();
	if ( aCurMsg != NULL ) {	// クローズボタンが押された
		mCandidatesView->Cancel();
	}
	return true;
}
//------------------------------------------------------------------------------
void	TACCandidatesWindow::SetCandidates(const list<string>& oCandidates)
{
	mCandidatesView->SetCandidates(oCandidates);
//	DBEXP("TACCandidatesWindow::SetCandidates", Bounds());
}
//------------------------------------------------------------------------------
//==============================================================================
TACFavoriteWindow::TACFavoriteWindow(BRect oFrame)
		: SUPER(oFrame, "ACFavorite", B_BORDERED_WINDOW_LOOK, B_FLOATING_APP_WINDOW_FEEL, B_AVOID_FRONT|B_AVOID_FOCUS)
{
	// ビュー作成
	mStrViewFavorite = new BStringView(BRect(0, 0, 1, 1), "Favorite", "");
	mStrViewFavorite->SetViewColor(FUC_COLOR_TOOL_TIP_WIN_BG);
	AddChild(mStrViewFavorite);
}
//------------------------------------------------------------------------------
TACFavoriteWindow::~TACFavoriteWindow()
{
}
//------------------------------------------------------------------------------
void	TACFavoriteWindow::SetFavorite(const string& oFavorite)
{
	BAutolock	aLock(this);

	if ( oFavorite == mFavorite )
		return;
	mFavorite = oFavorite;
	mStrViewFavorite->SetText((" "+mFavorite+" ").c_str());
	mStrViewFavorite->ResizeToPreferred();
	ResizeTo(mStrViewFavorite->Bounds().right,mStrViewFavorite->Bounds().bottom);
}
//------------------------------------------------------------------------------
string	TACFavoriteWindow::Favorite() const
{
	return mFavorite;
}
//------------------------------------------------------------------------------
//==============================================================================
TACTCMessageFilter::TACTCMessageFilter(BHandler* oTgt)
		: SUPER(B_KEY_DOWN)
{
	mTgt = oTgt;
}
//------------------------------------------------------------------------------
TACTCMessageFilter::~TACTCMessageFilter()
{
}
//------------------------------------------------------------------------------
filter_result	TACTCMessageFilter::Filter(BMessage* oMsg, BHandler** oTgt)
{
	*oTgt = mTgt;
	return B_DISPATCH_MESSAGE;
}
//==============================================================================
TACTextControl::TACTextControl(BRect oFrame, const char* oName, const char* oLabel,
		const char* oText, BMessage* oMsg, uint32 oResizingMode, uint32 oFlags)
		: SUPER(oFrame, oName, oLabel, oText, oMsg, oResizingMode, oFlags|B_DRAW_ON_CHILDREN)
{
	mPrevFocused = false;
	mMaxCandidates = DEFAULT_MAX_CANDIDATES;
	mCandidatesWindow = NULL;
	mFavoriteWindow = NULL;
	mFilter = NULL;
	smInstancesLocker.Lock();
	smInstances.push_back(this);	// ??: コンストラクト中に自分を追加して大丈夫かな？
	smInstancesLocker.Unlock();
}
//------------------------------------------------------------------------------
TACTextControl::~TACTextControl()
{
//	DBEXP("TACTextControl::~TACTextControl()", "");
	smInstancesLocker.Lock();
	smInstances.remove(this);
	DBEXP("TACCandidatesView::~TACCandidatesView()", smInstances.size());
	smInstancesLocker.Unlock();
}
//------------------------------------------------------------------------------
void	TACTextControl::AddCandidate(const string& oCandidate)
{
	list<string>::iterator	p;

	if ( oCandidate.length() <= mTrigger.length() )
		return;
	p = find(mCandidates.begin(), mCandidates.end(), oCandidate);
	if ( p == mCandidates.end() ) {
		// 追加
		mCandidates.push_front(oCandidate);
		if ( mCandidates.size() > mMaxCandidates ) {
			mCandidates.pop_back();
		}
	} else {
		// 順序変更
		mCandidates.push_front(*p);
		mCandidates.erase(p);
	}
	Save();
}
//------------------------------------------------------------------------------
void	TACTextControl::AddCandidates(const BMessage& oArc)
{
	int32	i;
	BString	aCandidate;

	i = 0;
	while ( oArc.FindString(ARC_NAME_CANDIDATE, i, &aCandidate) == B_OK ) {
		mCandidates.push_back(aCandidate.String());
		i++;
	}
}
//------------------------------------------------------------------------------
void	TACTextControl::AllAttached()
{
	// フィルタ
	mFilter = new TACTCMessageFilter(this);
	TextView()->AddFilter(mFilter);
}
//------------------------------------------------------------------------------
void	TACTextControl::AllDetached()
{
	TextView()->RemoveFilter(mFilter);
//	DBEXP("TACTextControl::AllDetached() - a", a);
	delete mFilter;
//	DBEXP("TACTextControl::AllDetached() - b", "");
	ManageFavoriteWindow(IsFavoriteWindowOpened(), false, "");
}
//------------------------------------------------------------------------------
void	TACTextControl::CandidateDetered(BMessage* oMsg)
{
	BString		aCandidate;

	if ( oMsg->FindString(MSG_NAME_CANDIDATE, &aCandidate) != B_OK )
		return;
	Deter(aCandidate.String());
}
//------------------------------------------------------------------------------
void	TACTextControl::CandidateWindowClosed()
{
	mCandidatesWindow = NULL;
}
//------------------------------------------------------------------------------
void	TACTextControl::Deter(const char* oCandidate)
{
	int32	aTgtPos;

	// 文字列の更新
	aTgtPos = SearchTargetEndPos();
	TextView()->Delete(string(Text()).rfind(mTrigger), aTgtPos);
	TextView()->Insert(oCandidate);
	// 候補の順序を更新
	AddCandidate(oCandidate);
}
//------------------------------------------------------------------------------
void	TACTextControl::DrawAfterChildren(BRect oUpdateRect)
{
//	DBEXP("TACTextControl::DrawAfterChildren", "");
	SUPER::DrawAfterChildren(oUpdateRect);
	if ( !Window()->IsActive() || TextView()->IsFocus() == mPrevFocused )
		return;
	if ( !TextView()->IsFocus() ) {
		ManageFavoriteWindow(IsFavoriteWindowOpened(), false, "");
		Learn();
	}
	mPrevFocused = TextView()->IsFocus();
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
既に入力されている文字列のみから判定する
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
status_t	TACTextControl::GetMatchCandidates(string* oFavorite, list<string>* oMatchCandidates) const
{
	int32				aTgtEnd;
	list<string>::const_iterator	p;
	string				aText;
	string				aTgt;
	string::size_type	aTriggerPos;

	aTgtEnd = SearchTargetEndPos();
	if ( aTgtEnd < 0 )
		return B_ERROR;
	aText = string(Text()).substr(0, aTgtEnd);
	aTriggerPos = aText.rfind(mTrigger);
	if ( aTriggerPos == string::npos )
		return B_ERROR;
	aTgt = aText.substr(aTriggerPos);
	p = mCandidates.begin();
	while ( p != mCandidates.end() ) {
		if ( p->substr(0, aTgt.length()) == aTgt && p->length() != aTgt.length() ) {
			if ( oMatchCandidates == NULL ) {
				// 最有力候補だけを返す
				*oFavorite = *p;
				return B_OK;
			} else {
				// 全リストを返す
				oMatchCandidates->push_back(*p);
			}
		}
		p++;
	}
	if ( oMatchCandidates == NULL || oMatchCandidates->empty() )
		return B_ERROR;
	return B_OK;
}
//------------------------------------------------------------------------------
void	TACTextControl::InsertCandidate(const string& oCandidate)
{
	int32	aCurrentPos = 0;
	list<string>::iterator	p;

	if ( oCandidate.length() <= mTrigger.length() )
		return;
	// 既に挿入位置よりも上位にいる場合は何もしない
	p = mCandidates.begin();
	while ( p != mCandidates.end() && aCurrentPos < INSERT_CANDIDATE_POSITION ) {
		if ( (*p) == oCandidate )
			return;
		p++;
		aCurrentPos++;
	}
	// 一旦削除
	p = find(mCandidates.begin(), mCandidates.end(), oCandidate);
	if ( p != mCandidates.end() ) {
		mCandidates.erase(p);
	}
	// 挿入
	p = mCandidates.begin();
	for ( int32 i = 0 ; i < INSERT_CANDIDATE_POSITION ; i++ ) {
		p++;
	}
	mCandidates.insert(p, oCandidate);
	Save();
}
//------------------------------------------------------------------------------
bool	TACTextControl::IsFavoriteWindowOpened() const
{
	return mFavoriteWindow != NULL;
}
//------------------------------------------------------------------------------
void	TACTextControl::KeyDown(const char* oBytes, int32 oNumBytes)
{
	bool	aCallChild = true;
	bool	aIsFavoriteOpen;			// 現在
	bool	aShiftKey;
	bool	aShouldFavoriteOpen = true;	// これから
	string	aFavorite;
	string	aKey(oBytes, oNumBytes);

//	DBEXP("TACTextControl::KeyDown", int32(aKey[0]));
	// 現状
	aIsFavoriteOpen = IsFavoriteWindowOpened();
	aShiftKey = modifiers() & B_SHIFT_KEY;
	// 処理
	if ( aKey[0] == B_ENTER && aShiftKey ) {
		aCallChild = false;
		// 最有力候補確定
		if ( aIsFavoriteOpen ) {
			Deter(mFavoriteWindow->Favorite().c_str());
		}
	} else if ( aKey[0] == B_ENTER ) {
		aShouldFavoriteOpen = false;
		Learn();
	} else if ( aKey[0] == B_ESCAPE ) {
		aCallChild = false;
		aShouldFavoriteOpen = false;
	} else if ( aKey[0] == B_TAB ) {
		aShouldFavoriteOpen = false;
		if ( aIsFavoriteOpen ) {
			aCallChild = false;
			ShowCandidatesWindow();
		}
	}
	// テキスト更新
	if ( aCallChild ) {
		TextView()->KeyDown(oBytes, oNumBytes);
	}
	// 最有力候補ウィンドウ
	if ( GetMatchCandidates(&aFavorite, NULL) != B_OK ) {
		aShouldFavoriteOpen = false;
	}
	ManageFavoriteWindow(aIsFavoriteOpen, aShouldFavoriteOpen, aFavorite);
}
//------------------------------------------------------------------------------
void	TACTextControl::Learn()
{
	list<string>::iterator	p;
	string	aTgt;
	string::size_type	aTriggerPos;

	if ( mLastAutoLearningText == Text() )
		return;
	mLastAutoLearningText = Text();
	// トップレベルターゲット
	aTriggerPos = mLastAutoLearningText.rfind(mTrigger);
	if ( aTriggerPos == string::npos )
		return;
	aTgt = mLastAutoLearningText.substr(aTriggerPos);
	p = find(mCandidates.begin(), mCandidates.end(), aTgt);
	if ( p == mCandidates.end() ) {
		AddCandidate(aTgt);
		return;
	}
	InsertCandidate(aTgt);
	// セカンドレベル以降
	for ( ;; ) {
		if ( aTriggerPos == 0 )
			return;
		aTriggerPos = mLastAutoLearningText.rfind(mTrigger, aTriggerPos-1);
		if ( aTriggerPos == string::npos )
			return;
		aTgt = mLastAutoLearningText.substr(aTriggerPos);
		p = find(mCandidates.begin(), mCandidates.end(), aTgt);
		InsertCandidate(aTgt);
		if ( p == mCandidates.end() )
			return;
	}
}
//------------------------------------------------------------------------------
void	TACTextControl::ManageFavoriteWindow(bool oIsFavoriteOpen, bool oShouldFavoriteOpen, const string& oFavorite)
{
	int32	aSelStart, aSelEnd;
	BRect	aWinFrame;

	// Close -> Close
	if ( !(oIsFavoriteOpen | oShouldFavoriteOpen) )
		return;
	// Open -> Close
	if ( oIsFavoriteOpen && !oShouldFavoriteOpen ) {
		close_window(reinterpret_cast<BWindow**>(&mFavoriteWindow));
		return;
	}
	if ( !oIsFavoriteOpen ) {
		// Close -> Open
		aWinFrame = ConvertToScreen(TextView()->Frame());
		TextView()->GetSelection(&aSelStart, &aSelEnd);
		if ( aSelStart == aSelEnd ) {
			aWinFrame.left += TextView()->PointAt(aSelStart).x;
		}
		aWinFrame.top += TextView()->LineHeight()+(Bounds().bottom-TextView()->Bounds().bottom);
		mFavoriteWindow = new TACFavoriteWindow(aWinFrame);
		mFavoriteWindow->IntelligentMoveTo(aWinFrame.LeftTop());
		mFavoriteWindow->Appear();
	} else {
		// Open -> Open
	}
	// xxx -> Open
	mFavoriteWindow->SetFavorite(oFavorite);
}
//------------------------------------------------------------------------------
void	TACTextControl::MateCandidatesUpdated(BMessage* oMsg)
{
	BString		aStorageFileName;

	if ( oMsg->FindString(MSG_NAME_STORAGE_FILE_NAME, &aStorageFileName) != B_OK )
		return;
	if ( mCandidatesStorageFileName != aStorageFileName.String() )
		return;
	mCandidates.clear();
	AddCandidates(*oMsg);
}
//------------------------------------------------------------------------------
void	TACTextControl::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case CANDIDATE_DETERED:
		CandidateDetered(oMsg);
		break;
	case CANDIDATE_WINDOW_CLOSED:
		CandidateWindowClosed();
		break;
	case MATE_CANDIDATES_UPDATED:
		MateCandidatesUpdated(oMsg);
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
status_t	TACTextControl::Save() const
{
	list<string>::const_iterator			p;
	list<TACTextControl*>::const_iterator	q;
	BFile		aFile;
	BMessage	aArc;
	BMessage	aSendMsg;

	if ( mCandidatesStorageFileName.empty() )
		return B_ERROR;
	// パック
	p = mCandidates.begin();
	while ( p != mCandidates.end() ) {
		aArc.AddString(ARC_NAME_CANDIDATE, p->c_str());
		p++;
	}
	// 仲間に変更を通知
	aSendMsg = aArc;
	aSendMsg.what = MATE_CANDIDATES_UPDATED;
	aSendMsg.AddString(MSG_NAME_STORAGE_FILE_NAME, mCandidatesStorageFileName.c_str());
	smInstancesLocker.Lock();
	q = smInstances.begin();
	while ( q != smInstances.end() ) {
		if ( *q != this ) {
			if ( (*q)->Looper() != NULL ) {
				// 走行中のスレッドから呼び出されている可能性があるので直接メンバを変更できない
				BMessenger(*q).SendMessage(&aSendMsg);
			} else {
				if ( mCandidatesStorageFileName == (*q)->mCandidatesStorageFileName ) {
					(*q)->mCandidates = mCandidates;
				}
			}
		}
		q++;
	}
	smInstancesLocker.Unlock();
	// 保存
	aFile.SetTo(mCandidatesStorageFileName.c_str(), B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	return aArc.Flatten(&aFile);
}
//------------------------------------------------------------------------------
int32	TACTextControl::SearchTargetEndPos() const
{
	int32	aTgtStart, aTgtEnd;

	// カーソル位置までの文字列を検索対象とする
	TextView()->GetSelection(&aTgtStart, &aTgtEnd);
	if ( aTgtStart != aTgtEnd )	// 選択されている場合は補完対象外
		return -1;
	return aTgtEnd;
}
//------------------------------------------------------------------------------
status_t	TACTextControl::SetMaxCandidates(uint32 oMax)
{
	if ( oMax < 1 )
		return B_ERROR;
	mMaxCandidates = oMax;
	// 余分な候補を削除
	while ( mCandidates.size() > mMaxCandidates ) {
		mCandidates.pop_back();
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TACTextControl::SetTo(const string& oPath)
{
	BFile		aFile;
	BMessage	aArc;

	// 初期化
	mCandidatesStorageFileName = oPath;
	mCandidates.clear();
	// 読み込み
	aFile.SetTo(mCandidatesStorageFileName.c_str(), B_READ_ONLY);
	if ( aArc.Unflatten(&aFile) != B_OK )
		return B_ERROR;
	AddCandidates(aArc);
	return B_OK;
}
//------------------------------------------------------------------------------
void	TACTextControl::SetTrigger(const string& oTrigger)
{
	mTrigger = oTrigger;
}
//------------------------------------------------------------------------------
void	TACTextControl::ShowCandidatesWindow()
{
	int32	aSelStart, aSelEnd;
	list<string>	aMatchCandidates;
	BRect	aWinFrame;

	if ( mCandidatesWindow != NULL )
		return;
	// ウィンドウに表示する候補の選定
	if ( GetMatchCandidates(NULL, &aMatchCandidates) != B_OK )
		return;
	// ウィンドウ表示
	aWinFrame = ConvertToScreen(TextView()->Frame());
	TextView()->GetSelection(&aSelStart, &aSelEnd);
	if ( aSelStart == aSelEnd ) {
		aWinFrame.left += TextView()->PointAt(aSelStart).x;
	}
	aWinFrame.top += TextView()->LineHeight()+(Bounds().bottom-TextView()->Bounds().bottom)
			+FUC_HEIGHT_WIN_TAB;
	mCandidatesWindow = new TACCandidatesWindow(aWinFrame, this);
	mCandidatesWindow->IntelligentMoveTo(aWinFrame.LeftTop());
	mCandidatesWindow->Appear();
	mCandidatesWindow->SetCandidates(aMatchCandidates);
}
//------------------------------------------------------------------------------
//==============================================================================
