/*==============================================================================
		メッセージ遅延配達		for Zeta		Ver 1.00
		Copyright (C) 2002-2003 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/12/07 (Sun) 開発着手。
 1.00 | 2003/12/07 (Sun) add 時代のクラスを元に構築。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "DelaySender.h"
//------------------------------------------------------------------------------
// BeOS
#include <Looper.h>
// C++
// Add2
#include <add2/MiscDebug.h>
// Proj
//------------------------------------------------------------------------------
const uint32	DELAY_SENDER_TIMEOUT = 'DSto';
//------------------------------------------------------------------------------
const char		MSGNAME_DELAY_SENDER_MSG_RUNNER_ID[] = "DlySMRID";
const char		MSGNAME_DELAY_SENDER_MSG_WHAT[] = "DlySMW";
//==============================================================================
TDelaySender::TDelaySender(const char* oName)
		: SUPER(oName)
{
	mMsgRunnerIDCnt = 0;
	mDelay = 0;
}
//------------------------------------------------------------------------------
TDelaySender::~TDelaySender()
{
	map<uint32,TDelaySendInfo>::iterator	p;

	p = mInfoMap.begin();
	while ( p != mInfoMap.end() ) {
		delete p->second.mMsgRunner;
		p++;
	}
}
//------------------------------------------------------------------------------
void	TDelaySender::DelaySenderTimeout(BMessage* oMsg)
{
	int32	aMsgRunnerID;
	uint32	aMsgWhat;
	map<uint32,TDelaySendInfo>::iterator	p;

	if ( oMsg->FindInt32(MSGNAME_DELAY_SENDER_MSG_RUNNER_ID,&aMsgRunnerID) != B_OK )
		return;
	if ( oMsg->FindInt32(MSGNAME_DELAY_SENDER_MSG_WHAT,reinterpret_cast<int32*>(&aMsgWhat)) != B_OK )
		return;
//	DBEXP("TDelaySender::DelaySenderTimeout - 送信要求受け取り",aMsgWhat);
	p = mInfoMap.find(aMsgWhat);
	ASSERT(p == mInfoMap.end(),"TDelaySender::DelaySenderTimeout - 送信すべきメッセージ無し");
	if ( p->second.mMsgRunnerID != aMsgRunnerID )	// 既に削除された古いメッセージランナーから送られてきたメッセージ
		return;
	// メッセージ送信
	mTarget.SendMessage(&p->second.mMsg);
	// 後始末
	delete p->second.mMsgRunner;
	mInfoMap.erase(p);
//	DBEXP("TDelaySender::DelaySenderTimeout - 送信した",aMsgWhat);
}
//------------------------------------------------------------------------------
void	TDelaySender::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case DELAY_SENDER_TIMEOUT:
		DelaySenderTimeout(oMsg);
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
status_t	TDelaySender::SendMessage(BMessage* oMsg)
{
	map<uint32,TDelaySendInfo>::iterator	p;
	BMessage		aMsgRunnerMsg(DELAY_SENDER_TIMEOUT);	// 遅配時刻を通知するメッセンジャーが持つメッセージ
	TDelaySendInfo	aDelaySendInfo;

	aMsgRunnerMsg.AddInt32(MSGNAME_DELAY_SENDER_MSG_RUNNER_ID,mMsgRunnerIDCnt);
	aMsgRunnerMsg.AddInt32(MSGNAME_DELAY_SENDER_MSG_WHAT,oMsg->what);
	p = mInfoMap.find(oMsg->what);
	if ( p == mInfoMap.end() ) {	// 新しいメッセージ
		aDelaySendInfo.mMsgRunnerID = mMsgRunnerIDCnt;
		aDelaySendInfo.mMsg = *oMsg;
		aDelaySendInfo.mMsgRunner = new BMessageRunner(BMessenger(this),&aMsgRunnerMsg,mDelay,1);
		mInfoMap[oMsg->what] = aDelaySendInfo;
	} else {	// 既にキューに入っているメッセージ
		delete p->second.mMsgRunner;	// 古いメッセージランナーを削除
		p->second.mMsgRunnerID = mMsgRunnerIDCnt;
		p->second.mMsg = *oMsg;	// what が同じでも中身が異なるかもしれないので最新のメッセージを採用
		p->second.mMsgRunner = new BMessageRunner(BMessenger(this),&aMsgRunnerMsg,mDelay,1);
	}
	mMsgRunnerIDCnt++;
	return B_OK;
}
//------------------------------------------------------------------------------
void	TDelaySender::SetDelay(bigtime_t oDelay)
{
	mDelay = oDelay;
}
//------------------------------------------------------------------------------
void	TDelaySender::SetTarget(BMessenger oMsger)
{
	BLooper*	aTargetLooper;

	// 今までの送信先から消える
	mTarget.Target(&aTargetLooper);
	if ( aTargetLooper != NULL ) {
		aTargetLooper->RemoveHandler(this);
	}
	// 更新
	mTarget = oMsger;
	// 送信先に寄生
	mTarget.Target(&aTargetLooper);
	if ( aTargetLooper != NULL ) {
		aTargetLooper->AddHandler(this);
	}
}
//==============================================================================
