/*==============================================================================
		エクスプローラビューコンポーネント		for Zeta		Ver 1.01
		Copyright (C) 2003-2004 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2004/01/28 (Wed) 開発着手。
 1.00 | 2004/01/28 (Wed) add 時代のクラスを元に構築。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

/*=== BOutlineListView Memo ======================================================
左の三角マークは、ListItem ではなく ListView 側で描画している
==============================================================================*/

//------------------------------------------------------------------------------
#include "ExplorerView.h"
//------------------------------------------------------------------------------
// BeOS
#include <Directory.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <Path.h>
// C++
// Add
#include <add2/DelaySender.h>
#include <add2/MiscDebug.h>
#include <add2/StringUtils.h>
// Proj
//------------------------------------------------------------------------------
const uint32	INIT_EVITEM_REQUESTED = 'EVie';		// まだ子持ちでないアイテムに子供を持たせるよう要求
const uint32	UPDATE_EVITEM_REQUESTED = 'EVue';	// 既に子持ちのアイテムを最新状態に更新するよう要求
//------------------------------------------------------------------------------
//==============================================================================
TEVItem::TEVItem(const string& oFolder, uint32 oLevel, bool oExpanded)
		: SUPER(extract_file_name(oFolder).c_str(), oLevel, oExpanded)
{
	mFolder = oFolder;
	mStatus = EVS_INIT;
}
//------------------------------------------------------------------------------
TEVItem::~TEVItem()
{
}
//------------------------------------------------------------------------------
void	TEVItem::DrawItem(BView* oOwner, BRect oItemRect, bool oDrawEverything)
{
	SUPER::DrawItem(oOwner, oItemRect, oDrawEverything);
}
//------------------------------------------------------------------------------
string	TEVItem::Folder() const
{
	return mFolder;
}
//------------------------------------------------------------------------------
void	TEVItem::SetStatus(TEVItemStatus oStatus)
{
	mStatus = oStatus;
}
//------------------------------------------------------------------------------
TEVItemStatus	TEVItem::Status() const
{
	return mStatus;
}
//------------------------------------------------------------------------------
void	TEVItem::Update(BView* oOwner, const BFont* oFont)
{
	BMessage	aSendMsg;

	SUPER::Update(oOwner, oFont);
	switch ( Status() ) {
	case EVS_INIT:
		aSendMsg.what = INIT_EVITEM_REQUESTED;
		aSendMsg.AddPointer(MSG_NAME_EVITEM_POINTER, this);
		BMessenger(oOwner).SendMessage(&aSendMsg);
//		DBEXP("TEVItem::Update", Text());
#if 0
		TDelaySender*	a = new TDelaySender("ds");
		a->SetTarget(BMessenger(oOwner));
		a->SetDelay(1*1000*1000);
		a->SendMessage(&aSendMsg);
#endif
		break;
	case EVS_OLD:
		aSendMsg.what = UPDATE_EVITEM_REQUESTED;
		aSendMsg.AddPointer(MSG_NAME_EVITEM_POINTER, this);
		BMessenger(oOwner).SendMessage(&aSendMsg);
		break;
	case EVS_LATEST:
		// 何もしない
		break;
	default:
		ASSERT(1, "TEVItem::Update - Bad state");
	}
}
//==============================================================================
TExplorerView::TExplorerView(BRect oFrame, const char* oName, list_view_type oType,
		uint32 oResizingMode, uint32 oFlags)
		: SUPER(oFrame, oName, oType, oResizingMode, oFlags)
{
	Init();
}
//------------------------------------------------------------------------------
TExplorerView::~TExplorerView()
{
	delete mVolumeRoster;
}
//------------------------------------------------------------------------------
bool	TExplorerView::AddItemRequested(const string& oFolder)
{
	return true;
}
//------------------------------------------------------------------------------
void	TExplorerView::AddRootTree()
{
	TEVItem*	aItem;

	if ( FullListIsEmpty() ) {
		aItem = new TEVItem(mRootFolder, 0, false);
		AddItem(aItem);	// 子持ち処理は、TEVItem::Update() 経由で自動的に行われる
	}
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''
oParentIdx は SUPER::IndexOf() 相当の値
   ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,, */
void	TExplorerView::AddTree(int32 oParentIdx)
{
	char		aVolumeName[B_FILE_NAME_LENGTH];
	string		aPath;
	BDirectory	aParentDir;
	BDirectory	aRootDir;
	BEntry		aEntry;
	BList		aItemList;
	BVolume		aVolume;
	TEVItem*	aItem;
	TEVItem*	aParentItem = dynamic_cast<TEVItem*>(ItemAt(oParentIdx));	// FullList... ではない

	// 一般フォルダ
	aParentDir.SetTo(aParentItem->Folder().c_str());
	aParentDir.Rewind();
	while ( aParentDir.GetNextEntry(&aEntry) == B_OK ) {
		if ( aEntry.IsDirectory() ) {
			aPath = BPath(&aEntry).Path();
			if ( AddItemRequested(aPath) ) {
				aItem = new TEVItem(aPath.c_str(), aParentItem->OutlineLevel()+1, false);
				aItemList.AddItem(aItem);
			}
		}
	}
	aItemList.SortItems(&CompareEVItem);
	AddList(&aItemList, FullListIndexOf(aParentItem)+1);	// oParentIdx を使ってはダメ
	// デスクトップ
	if ( aParentItem->Folder() == mDesktopFolder ) {
		aItemList.MakeEmpty();
		// ボリュームの追加（自力でソートしなくても、case insensitive で自動ソートされてる）
		mVolumeRoster->Rewind();
		while ( mVolumeRoster->GetNextVolume(&aVolume) == B_OK ) {
			if ( aVolume.IsPersistent() && aVolume.GetRootDirectory(&aRootDir) == B_OK ) {
				aItem = new TEVItem(BPath(&aRootDir, "").Path(), aParentItem->OutlineLevel()+1, false);
				if ( aVolume.GetName(aVolumeName) == B_OK ) {
					aItem->SetText(aVolumeName);
				}
				aItemList.AddItem(aItem);
			}
		}
		AddList(&aItemList, FullListIndexOf(aParentItem)+1);
	}
	// 完了
	AddTreeDone(oParentIdx);
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''
oParentIdx は追加したアイテム群の親で、SUPER::IndexOf() 相当の値
   ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,, */
void	TExplorerView::AddTreeDone(int32 oParentIdx)
{
}
//------------------------------------------------------------------------------
void	TExplorerView::AttachedToWindow()
{
	AddRootTree();
}
//------------------------------------------------------------------------------
int	TExplorerView::CompareEVItem(const void* oLHS, const void* oRHS)
{
//	const TEVItem*	aLHS = reinterpret_cast<const TEVItem*>(*static_cast<const void**>(oLHS));
//	const TEVItem*	aRHS = reinterpret_cast<const TEVItem*>(*static_cast<const void**>(oRHS));
	const TEVItem*	aLHS = reinterpret_cast<const TEVItem*>(*((const void**)oLHS));
	const TEVItem*	aRHS = reinterpret_cast<const TEVItem*>(*((const void**)oRHS));

	return CompareEVItemCore(aLHS, aRHS);
}
//------------------------------------------------------------------------------
int		TExplorerView::CompareEVItem(const BListItem* oLHS, const BListItem* oRHS)
{
	const TEVItem*	aLHS = dynamic_cast<const TEVItem*>(oLHS);
	const TEVItem*	aRHS = dynamic_cast<const TEVItem*>(oRHS);

	return CompareEVItemCore(aLHS, aRHS);
}
//------------------------------------------------------------------------------
int		TExplorerView::CompareEVItemCore(const TEVItem* oLHS, const TEVItem* oRHS)
{
	return strcasecmp(oLHS->Folder().c_str(), oRHS->Folder().c_str());
}
//------------------------------------------------------------------------------
TEVItem*	TExplorerView::FindItem(const string& oFolder)
{
	TEVItem*	aFoundItem = NULL;
	TEVItem*	aTmpItem;

	for ( int32 i = 0 ; i < FullListCountItems() ; i++ ) {
		aTmpItem = dynamic_cast<TEVItem*>(FullListItemAt(i));
		if ( aTmpItem->Folder() == oFolder ) {
			aFoundItem = aTmpItem;
			break;
		}
	}
	return aFoundItem;
}
//------------------------------------------------------------------------------
TEVItem*	TExplorerView::FindItem(const BList& oList, const string& oFolder)
{
	TEVItem*	aFoundItem = NULL;
	TEVItem*	aTmpItem;

	for ( int32 i = 0 ; i < oList.CountItems() ; i++ ) {
		aTmpItem = reinterpret_cast<TEVItem*>(oList.ItemAt(i));
		if ( aTmpItem->Folder() == oFolder ) {
			aFoundItem = aTmpItem;
			break;
		}
	}
	return aFoundItem;
}
//------------------------------------------------------------------------------
void	TExplorerView::Init()
{
	BPath	aDesktopPath;

	mVolumeRoster = new BVolumeRoster();
	if ( find_directory(B_DESKTOP_DIRECTORY, &aDesktopPath) == B_OK ) {
		mDesktopFolder = aDesktopPath.Path();
//		DBEXP("TExplorerView::Init - mDesktopFolder", mDesktopFolder);
	}
	// bool
	mShowRootFolder = true;
	// string
	mRootFolder = mDesktopFolder;
}
//------------------------------------------------------------------------------
void	TExplorerView::InitEVItemRequested(BMessage* oMsg)
{
	TEVItem*	aEVItem;

	if ( oMsg->FindPointer(MSG_NAME_EVITEM_POINTER, reinterpret_cast<void**>(&aEVItem)) != B_OK )
		return;
//	DBEXP("TExplorerView::InitEVItemRequested",aEVItem->Text());
	if ( IndexOf(aEVItem) >= 0 && aEVItem->Status() == EVS_INIT ) {
		DBEXP("found - TExplorerView::InitEVItemRequested", extract_file_name(aEVItem->Folder()));
		AddTree(IndexOf(aEVItem));
		aEVItem->SetStatus(EVS_LATEST);
	} else {
//		DBEXP("NOT found - TExplorerView::InitEVItemRequested", extract_file_name(aEVItem->Path()));
	}
}
//------------------------------------------------------------------------------
void	TExplorerView::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case INIT_EVITEM_REQUESTED:
		InitEVItemRequested(oMsg);
		break;
	case UPDATE_EVITEM_REQUESTED:
		UpdateEVItemRequested(oMsg);
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
status_t	TExplorerView::SetRootFolder(const string& oRootFolder)
{
	mRootFolder = oRootFolder;
	return B_OK;
}
//------------------------------------------------------------------------------
void	TExplorerView::Update(const string& oParentFolder)
{
	int32		aParentIdx;
	TEVItem*	aParentItem;	// 作成・削除などされたフォルダの親フォルダ

	aParentItem = FindItem(oParentFolder);
	if ( aParentItem == NULL )	// 存在しないか、ツリーに組み込まれていないフォルダ
		return;
	aParentIdx = IndexOf(aParentItem);
	if ( aParentIdx >= 0 ) {
		// 親は表示中……今すぐ更新
		UpdateTree(aParentIdx);
	} else {
		// 親は非表示……表示されるときに更新するので今は更新しない
		aParentItem->SetStatus(EVS_OLD);
	}
}
//------------------------------------------------------------------------------
void	TExplorerView::UpdateEVItemRequested(BMessage* oMsg)
{
	TEVItem*	aEVItem;

	if ( oMsg->FindPointer(MSG_NAME_EVITEM_POINTER, reinterpret_cast<void**>(&aEVItem)) != B_OK )
		return;
	DBEXP("TExplorerView::UpdateEVItemRequested",aEVItem->Text());
	if ( IndexOf(aEVItem) >= 0 && aEVItem->Status() == EVS_OLD ) {
		DBEXP("found - TExplorerView::UpdateEVItemRequested", extract_file_name(aEVItem->Folder()));
		Update(aEVItem->Folder());
	} else {
		DBEXP("NOT found - TExplorerView::UpdateEVItemRequested", extract_file_name(aEVItem->Folder()));
	}
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''
oParentIdx は SUPER::IndexOf() 相当の値
   ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,, */
void	TExplorerView::UpdateTree(int32 oParentIdx)
{
	bool		aNeedSort = false;
	int32		aItemListIdx;
	int32		aTreeIdx;		// FullList...
	BDirectory	aParentDir;
	BEntry		aEntry;
	BList		aItemList;
	TEVItem*	aItem;
	TEVItem*	aParentItem = dynamic_cast<TEVItem*>(ItemAt(oParentIdx));	// FullList... ではない
	TEVItem*	aTreeItem;

	DBEXP("TExplorerView::UpdateTree", aParentItem->Folder());
	// 一般フォルダ
	aParentDir.SetTo(aParentItem->Folder().c_str());
	aParentDir.Rewind();
	while ( aParentDir.GetNextEntry(&aEntry) == B_OK ) {
		if ( aEntry.IsDirectory() ) {
			aItem = new TEVItem(BPath(&aEntry).Path(), aParentItem->OutlineLevel()+1, false);
			aItemList.AddItem(aItem);
//			DBEXP("dir - TExplorerView::UpdateTree", BPath(&aEntry).Path());
		}
	}
	aItemList.SortItems(&CompareEVItem);
	// 削除されたフォルダ
	for ( aTreeIdx = CountItemsUnder(aParentItem, true)-1 ; aTreeIdx >= 0 ; aTreeIdx-- ) {
		aTreeItem = dynamic_cast<TEVItem*>(ItemUnderAt(aParentItem, true, aTreeIdx));
		if ( FindItem(aItemList, aTreeItem->Folder()) == NULL ) {
			// aTreeItem は削除されたアイテム
			DBEXP("<< removed - TExplorerView::UpdateTree", aTreeItem->Folder());
			RemoveItem(aTreeItem);
			delete aTreeItem;
		}
	}
	// 作成されたフォルダ
	for ( aItemListIdx = 0 ; aItemListIdx < aItemList.CountItems() ; aItemListIdx++ ) {
		aItem = reinterpret_cast<TEVItem*>(aItemList.ItemAt(aItemListIdx));
		if ( FindItem(aItem->Folder()) == NULL ) {
			// aItem は新しく加わったアイテム
			DBEXP(">> added - TExplorerView::UpdateTree", aItem->Folder());
			AddItem(new TEVItem(aItem->Folder(), aItem->OutlineLevel(), aItem->IsExpanded()), FullListIndexOf(aParentItem)+1);
			aNeedSort = true;
		}
	}
	if ( aNeedSort ) {
		SortItemsUnder(aParentItem, true, &CompareEVItem);
	}
	// メモリ解放
	for ( int32 i = 0 ; i < aItemList.CountItems() ; i++ ) {
		delete reinterpret_cast<TEVItem*>(aItemList.ItemAt(i));
	}
	aItemList.MakeEmpty();
	// 再描画
	InvalidateItem(oParentIdx);
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//==============================================================================
