#ifndef	___GUIBase_H_
#define	___GUIBase_H_
//------------------------------------------------------------------------------
// BeOS
#include <View.h>
#include <Window.h>
// Add2
#include "MouseDownManager.h"
//------------------------------------------------------------------------------
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
基本的なビュー
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TBasicView : public BView, public TMouseDownManager
{
public:
	TBasicView(BRect oFrame, const char* oName,uint32 oResizingMode, uint32 oFlags);
	TBasicView(BMessage* oArchive);
	virtual ~TBasicView();

	virtual void	IAPChanged(BMessage* oMsg);
	virtual void	LanguageChanged();

protected:
	virtual void	MessageReceived(BMessage* oMsg);
	virtual void	MouseDown(BPoint oPoint);
	virtual void	MouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg);
	virtual void	MouseUp(BPoint oPoint);

private:
	typedef	BView	SUPER;

};
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
ダイアログ用のビュー
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TDialogView : public TBasicView
{
public:
	TDialogView(BRect oFrame, const char* oName, uint32 oResizingMode = B_FOLLOW_NONE, uint32 oFlags = B_WILL_DRAW);
	TDialogView(BMessage* oArchive);
	virtual ~TDialogView();

	void			SetTargetToThis(BView* oParent, bool oForce = false) const;	// メッセージ受け取り先を自分にする

protected:
	virtual void	ShortCutDown(int32 oChar, int32 oFunc);

private:
	typedef	TBasicView	SUPER;
	
	friend class	TDialogWindow;	// ShortCutDown() を呼び出してもらうため

	status_t	ApplyShortCut(const BView* oTgtView,int32 oChar, int32 oFunc) const;	// 文字ショートカットの適用（再帰的）
};
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
基本的なウィンドウ
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TBasicWindow : public BWindow
{
public:
	TBasicWindow(BRect oFrame, const char* oTitle, window_type oType, uint32 oFlags, uint32 oWorkspace = B_CURRENT_WORKSPACE);
	TBasicWindow(BRect oFrame, const char* oTitle, window_look oLook, window_feel oFeel, uint32 oFlags, uint32 oWorkspace = B_CURRENT_WORKSPACE);
	~TBasicWindow();

	virtual void	Appear();							// 隠れていたら Show() し、表示されていたら前面に出す
	virtual void	IAPChanged(BMessage* oMsg);
	BPoint			IntelligentMoveTo(BPoint oPoint);	// 画面からはみ出ない範囲で移動
	BPoint			IntelligentCentering();				// センタリング
	virtual void	LanguageChanged();
	virtual bool	QuitRequested();

protected:
	virtual void	MessageReceived(BMessage* oMsg);

private:
	typedef	BWindow	SUPER;
	
	void			Init();								// 初期化
	void			TransferToView(BMessage* oMsg);
};
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
ダイアログ用のウィンドウ
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TDialogWindow : public TBasicWindow
{
public:
	TDialogWindow(BRect oFrame, const char* oTitle, window_type oType = B_TITLED_WINDOW, uint32 oFlags = B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS, uint32 oWorkspace = B_CURRENT_WORKSPACE);
	~TDialogWindow();

	BMessage*		Message() const;						// デフォルトメッセージ
	virtual void	Hide();
	status_t		Invoke(BMessage* oMsg = NULL);			// メッセージを送付（メッセージの所有権は呼び出し元のまま）
	status_t		SetMessage(BMessage* oMsg);				// 送付するメッセージのデフォルトを設定
			// 引数のメッセージの所有権は TDialogWindow に移るので呼び出し側では削除しないこと
	void			SetTarget(const BMessenger& oTgtMsger);	// メッセージ送付先を設定
	virtual void	Show();
	BMessenger		Target() const;							// メッセージ送付先を返す

protected:
	virtual void	KeyDown(BMessage* oMsg);				// コンポーネントショートカット処理
	virtual void	MessageReceived(BMessage* oMsg);

	BMessenger		mTgtMsger;								// メッセージ送付先（他のウィンドウなど）

private:
	typedef	TBasicWindow	SUPER;

	BMessage*		mDefaultMsg;							// 送付するメッセージのデフォルト
};
//==============================================================================
#endif	// ___GUIBase_H_
