/*==============================================================================
		ファイル履歴コンポーネント		for Zeta		Ver 1.02
		Copyright (C) 2001-2004 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/11/19 (Wed) 開発着手。
 1.00 | 2003/11/19 (Wed) add 時代のクラスを元に構築。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "MRUF.h"
//------------------------------------------------------------------------------
// BeOS
#include <MenuItem.h>
// C++
#include <cstring>
// Add2
#include <add2/BeConstants.h>
#include <add2/MiscUtils.h>
#include <add2/StringUtils.h>
// Proj
//------------------------------------------------------------------------------
//==============================================================================
TMRUF::TMRUF()
		: SUPER()
{
	mShowSC = false;
	mTgtBase = 0;
	mTriggerBase = 0;
	mMaxMRUF = 10;
	mTgtHandler = NULL;
	mTgtMenu = NULL;
	mMItemMsg.what = B_REFS_RECEIVED;
}
//------------------------------------------------------------------------------
TMRUF::~TMRUF()
{
}
//------------------------------------------------------------------------------
void	TMRUF::AddMRUF(const char* oPath)
{
	char*		aCStr;
	const char*	aConstCStr;
	int32		aIdx;

	aIdx = FindMRUF(oPath);
	if ( aIdx < 0 ) {	// リストにないファイル
		aCStr = new char [strlen(oPath)+1];
		strcpy(aCStr, oPath);
		mMRUFs.insert(mMRUFs.begin(), aCStr);
		DeleteExtraMRUF();
	} else {	// リストにあるファイル
		aConstCStr = RemoveMRUF(aIdx);
		mMRUFs.insert(mMRUFs.begin(), aConstCStr);
	}
}
//------------------------------------------------------------------------------
status_t	TMRUF::Archive(BMessage* oArc, bool oDeep) const
{
	int			i;
	status_t	aResult = B_OK;

	SUPER::Archive(oArc, oDeep);
	for ( i = CountMRUFs()-1 ; i >= 0  ; i-- ) {
		aResult |= oArc->AddString(MRUF_ARCNAME_MRUF, MRUFAt(i));
	}
	return aResult;
}
//------------------------------------------------------------------------------
void	TMRUF::ClearMRUF()
{
	while ( CountMRUFs() > 0 ) {
		delete [] RemoveMRUF(static_cast<int32>(0));
	}
}
//------------------------------------------------------------------------------
int32	TMRUF::CountMRUFs() const
{
	return static_cast<int32>(mMRUFs.size());
}
//------------------------------------------------------------------------------
void	TMRUF::DeleteExtraMRUF()
{
	while ( CountMRUFs() > mMaxMRUF ) {
		delete [] RemoveMRUF(CountMRUFs()-1);
	}
}
//------------------------------------------------------------------------------
int32	TMRUF::FindMRUF(const char* oPath)
{
	int		i;
	int32	aIdx = -1;

	for ( i = 0 ; i < CountMRUFs() ; i++ ) {
		if ( strcmp(MRUFAt(i), oPath) == 0 ) {
			aIdx = i;
			break;
		}
	}
	return aIdx;
}
//------------------------------------------------------------------------------
BMenu*	TMRUF::GenerateMenu()
{
	entry_ref	aERef;
	string		aStr;
	BMenuItem*	aMItem;
	BMessage*	aMsg;

	if ( mTgtMenu == NULL )
		return NULL;
	// 古い履歴メニューを削除
	while ( mTgtMenu->CountItems() > mTgtBase && dynamic_cast<BSeparatorItem*>(mTgtMenu->ItemAt(mTgtBase)) == NULL ) {
		delete mTgtMenu->RemoveItem(mTgtBase);
	}
	// 新しい履歴メニューを作成
	for ( int i = 0 ; i < CountMRUFs() ; i++ ) {
		get_ref_for_path(MRUFAt(i), &aERef);
		aMsg = new BMessage(mMItemMsg);
		aMsg->AddRef(B_MSG_NAME_REFS, &aERef);
		aStr = "";
		if ( i+mTriggerBase < 10 ) {
			aStr += static_cast<char>(i+mTriggerBase+'0');
		} else {
			aStr += static_cast<char>(i-10+mTriggerBase+'A');
		}
		aStr = merge_accelerator_key(mAccFormat, aStr, extract_file_name(MRUFAt(i)));
		if ( mShowSC && i+mTriggerBase < 10 ) {
			aMItem = new BMenuItem(aStr.c_str(), aMsg, i+mTriggerBase+'0', B_SHIFT_KEY);
		} else {
			aMItem = new BMenuItem(aStr.c_str(), aMsg);
		}
		aMItem->SetTarget(mTgtHandler);
		mTgtMenu->AddItem(aMItem, mTgtBase+i);
	}
//	mxTgtMenu->SetTargetForItems(mxTgtHandler);	// 他のメニューアイテムのターゲットまで設定されてしまう
	return mTgtMenu;
}
//------------------------------------------------------------------------------
const char*	TMRUF::MRUFAt(int32 oIdx) const
{
	return mMRUFs[oIdx];
}
//------------------------------------------------------------------------------
const char*	TMRUF::RemoveMRUF(int32 oIdx)
{
	const char*	aRemovedMRUF;

	if ( oIdx < 0 || oIdx >= CountMRUFs() )
		return NULL;
	aRemovedMRUF = mMRUFs[oIdx];
	mMRUFs.erase(mMRUFs.begin()+oIdx);
	return aRemovedMRUF;
}
//------------------------------------------------------------------------------
const char*	TMRUF::RemoveMRUF(const char* oPath)
{
	int32	aIdx;

	aIdx = FindMRUF(oPath);
	if ( aIdx == -1 ) {
		return NULL;
	}
	return RemoveMRUF(aIdx);
}
//------------------------------------------------------------------------------
void	TMRUF::SetAccFormat(const string& oAccFormat)
{
	mAccFormat = oAccFormat;
}
//------------------------------------------------------------------------------
void	TMRUF::SetMaxMRUF(int32 oMaxMRUF)
{
	if ( oMaxMRUF < 1 )
		oMaxMRUF = 1;
	if ( oMaxMRUF > 20 )
		oMaxMRUF = 20;
	mMaxMRUF = oMaxMRUF;
	DeleteExtraMRUF();
}
//------------------------------------------------------------------------------
void	TMRUF::SetMessage(BMessage* oMsg)
{
	mMItemMsg = *oMsg;	// 所有権移動
	delete oMsg;
}
//------------------------------------------------------------------------------
void	TMRUF::SetShowSC(bool oShow)
{
	mShowSC = oShow;
}
//------------------------------------------------------------------------------
void	TMRUF::SetTgtBase(int32 oTgtBase)
{
	mTgtBase = oTgtBase;
}
//------------------------------------------------------------------------------
void	TMRUF::SetTgtHandler(BHandler* oTgtHandler)
{
	mTgtHandler = oTgtHandler;
}
//------------------------------------------------------------------------------
void	TMRUF::SetTriggerBase(int32 oTriggerBase)
{
	mTriggerBase = oTriggerBase;
}
//------------------------------------------------------------------------------
void	TMRUF::SetTgtMenu(BMenu* oMenu)
{
	mTgtMenu = oMenu;
}
//------------------------------------------------------------------------------
status_t	TMRUF::Unarchive(const BMessage& oArchive)
{
	int			i = 0;
	const char*	aFNameRef;
	entry_ref	aRef;

	SUPER::Unarchive(oArchive);
	ClearMRUF();
	while ( oArchive.FindString(MRUF_ARCNAME_MRUF, i, &aFNameRef) == B_OK ) {
		AddMRUF(strdup(aFNameRef));
		i++;
	}
	return B_OK;
}
//==============================================================================
