/*==============================================================================
		BView::MouseDown() を細かく振り分ける	for Zeta	Ver 1.11
		Copyright (C) 2004 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2004/02/07 (Sat) 開発着手。
 1.00 | 2004/02/07 (Sat) TBasicView を元に構築。
 1.10 | 2004/02/07 (Sat) ドラッグ開始・途中・終了、マウスアップを判定できるようにした。
==============================================================================*/

/*=== Memo =====================================================================
クリックすると、Down → Click → Up の順で発効
ダブルクリック以降は Down/Up は発効しない
ドラッグすると、Down → DragBegin → Dragging → DragEnd → Up の順で発効
==============================================================================*/

//------------------------------------------------------------------------------
#include "MouseDownManager.h"
//------------------------------------------------------------------------------
// BeOS
#include <Window.h>
// C++
// Add2
#include <add2/BeConstants.h>
//------------------------------------------------------------------------------
const int32		SEQ_LEN = 3;					// 何回連続まで区別するか
//------------------------------------------------------------------------------
const float		CLICK_JUDGE_DISTANCE = 16.0;	// クリックかドラッグか判断する移動距離
//------------------------------------------------------------------------------
//==============================================================================
TMouseDownManager::TMouseDownManager(BView* oTgtView)
{
	mLastDrag = false;
	mLastButtons = 0;
	mLastClicks = 0;
	mLastPoint = B_ORIGIN;
	mTgtView = oTgtView;
}
//------------------------------------------------------------------------------
TMouseDownManager::~TMouseDownManager()
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::ManageMouseDown(BPoint oPoint)
{
	int32			aButtons;
	int32			aClicks;

	// ボタンと回数の処理
	mLastDrag = false;
	mTgtView->Window()->CurrentMessage()->FindInt32(B_MSG_NAME_BUTTONS, &aButtons);
	mTgtView->Window()->CurrentMessage()->FindInt32(B_MSG_NAME_CLICKS, &aClicks);
	mLastClicks = aClicks;
	mLastButtons = aButtons;
	mLastPoint = oPoint;
	// マウスダウン
	if ( aClicks%SEQ_LEN == 1 ) {
		switch ( aButtons ) {
		case B_PRIMARY_MOUSE_BUTTON:
			MouseDownLeft(oPoint);
			break;
		case B_SECONDARY_MOUSE_BUTTON:
			MouseDownRight(oPoint);
			break;
		}
	}
	mTgtView->SetMouseEventMask(B_POINTER_EVENTS, B_NO_POINTER_HISTORY);
}
//------------------------------------------------------------------------------
void	TMouseDownManager::ManageMouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg)
{
	BPoint	aPointDiff;

	if ( mLastButtons == 0 )
		return;
	if ( !mLastDrag ) {
		// ドラッグ開始チェック
		aPointDiff = oPoint-mLastPoint;
		if ( aPointDiff.x*aPointDiff.x+aPointDiff.y*aPointDiff.y > CLICK_JUDGE_DISTANCE ) {
			mLastDrag = true;
			switch ( mLastButtons ) {
			case B_PRIMARY_MOUSE_BUTTON:
				MouseDragBeginLeft(mLastPoint, oPoint);
				break;
			case B_SECONDARY_MOUSE_BUTTON:
				MouseDragBeginRight(mLastPoint, oPoint);
				break;
			}
		}
	}
	if ( mLastDrag ) {
		// ドラッグ中
		switch ( mLastButtons ) {
		case B_PRIMARY_MOUSE_BUTTON:
			MouseDraggingLeft(mLastPoint, oPoint);
			break;
		case B_SECONDARY_MOUSE_BUTTON:
			MouseDraggingRight(mLastPoint, oPoint);
			break;
		}
	}
}
//------------------------------------------------------------------------------
void	TMouseDownManager::ManageMouseUp(BPoint oPoint)
{
	if ( mLastButtons == 0 )
		return;
	if ( !mLastDrag ) {
		// クリックチェック
		switch ( mLastClicks%SEQ_LEN ) {
		case 1:
			switch ( mLastButtons ) {
			case B_PRIMARY_MOUSE_BUTTON:
				MouseClickLeftSingle(oPoint);
				break;
			case B_SECONDARY_MOUSE_BUTTON:
				MouseClickRightSingle(oPoint);
				break;
			}
			break;
		case 2:
			switch ( mLastButtons ) {
			case B_PRIMARY_MOUSE_BUTTON:
				MouseClickLeftDouble(oPoint);
				break;
			case B_SECONDARY_MOUSE_BUTTON:
				MouseClickRightDouble(oPoint);
				break;
			}
			break;
		}
	} else {
		// ドラッグ終了
		switch ( mLastButtons ) {
		case B_PRIMARY_MOUSE_BUTTON:
			MouseDragEndLeft(mLastPoint, oPoint);
			break;
		case B_SECONDARY_MOUSE_BUTTON:
			MouseDragEndRight(mLastPoint, oPoint);
			break;
		}
	}
	// マウスアップ
	if ( mLastClicks%SEQ_LEN == 1 ) {
		switch ( mLastButtons ) {
		case B_PRIMARY_MOUSE_BUTTON:
			MouseUpLeft(oPoint);
			break;
		case B_SECONDARY_MOUSE_BUTTON:
			MouseUpRight(oPoint);
			break;
		}
	}
	// クリア
	mLastButtons = 0;
	mLastDrag = false;
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseClickLeftDouble(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseClickLeftSingle(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseClickRightDouble(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseClickRightSingle(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDownLeft(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDownRight(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDragBeginLeft(BPoint oBeginPoint, BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDragBeginRight(BPoint oBeginPoint, BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDragEndLeft(BPoint oBeginPoint, BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDragEndRight(BPoint oBeginPoint, BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDraggingLeft(BPoint oBeginPoint, BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseDraggingRight(BPoint oBeginPoint, BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseUpLeft(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
void	TMouseDownManager::MouseUpRight(BPoint oPoint)
{
}
//------------------------------------------------------------------------------
//==============================================================================
