/*==============================================================================
		スクロールバー付きタブビュー	for Zeta	Ver 1.20
		Copyright (C) 2004 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2004/02/04 (Wed) 開発着手。
 1.00 | 2004/02/07 (Sat) オリジナルバージョン。
(1.11)| 2004/02/08 (Sun) スクロールバーではなくボタンでスクロールするようにした。
 1.20 | 2004/05/05 (Wed) 選択変更時にメッセージを飛ばせるようにした。
==============================================================================*/

/*=== Memo =====================================================================
なんかスクロールバーをつけたら終了時に落ちるようになったんだけど。いつの間にか直ったみたい。
不要 タブをつかんでスクロールできるようにしようかと思ったが、すぐ下にスクロールバーがあるのでその機能は不要。むしろ違和感がある。
==============================================================================*/

//------------------------------------------------------------------------------
#include "ScrollTabView.h"
//------------------------------------------------------------------------------
// BeOS
// C++
// Add2
#include <add2/MiscDebug.h>
//------------------------------------------------------------------------------
const uint32	TBUTTON_SCRL_LEFT_CLICKED = 'tscl';
const uint32	TBUTTON_SCRL_RIGHT_CLICKED = 'tscr';
//------------------------------------------------------------------------------
const float		WIDTH_SCRL_BUTTON = 15.0;
//==============================================================================
TScrollTabView::TScrollTabView(BRect oFrame, const char* oName, button_width oWidth,
		uint32 oResizingMode, uint32 oFlags)
		: SUPER(oFrame, oName, oWidth, oResizingMode, oFlags), TMouseDownManager(this)
{
	BRect	aButtonFrame(0, 0, WIDTH_SCRL_BUTTON, TabHeight());

	mTButtonLeft = new TToolButton(aButtonFrame, "ScrlButtonLeft", NULL, "<", new BMessage(TBUTTON_SCRL_LEFT_CLICKED));
	AddChild(mTButtonLeft);
	mTButtonRight = new TToolButton(aButtonFrame.OffsetToCopy(Bounds().Width()-aButtonFrame.Width()+1, 0), "ScrlButtonRight", NULL, ">", new BMessage(TBUTTON_SCRL_RIGHT_CLICKED));
	mTButtonRight->SetResizingMode(B_FOLLOW_RIGHT);
	AddChild(mTButtonRight);
	// 初期化
	mShowScrlButtons = false;
	mLastSelection = -1;
	mScrlPos = mScrlPosOnDragBegin = 0.0;
}
//------------------------------------------------------------------------------
TScrollTabView::~TScrollTabView()
{
}
//------------------------------------------------------------------------------
void	TScrollTabView::AddTab(BView* oTgt, BTab* oTab)
{
	SUPER::AddTab(oTgt, oTab);
	UpdateScrollButtons();
	CheckSelectionChange();
}
//------------------------------------------------------------------------------
void	TScrollTabView::AllAttached()
{
	SUPER::AllAttached();
	mTButtonLeft->SetTarget(this);
	mTButtonRight->SetTarget(this);
}
//------------------------------------------------------------------------------
void	TScrollTabView::CheckSelectionChange()
{
	if ( Selection() == mLastSelection )
		return;
	mLastSelection = Selection();
	Invoke();
}
//------------------------------------------------------------------------------
void	TScrollTabView::FrameResized(float oWidth, float oHeight)
{
	UpdateScrollButtons();
}
//------------------------------------------------------------------------------
void	TScrollTabView::InvalidateTabs()
{
	BRect	aFrame = Bounds();

	aFrame.bottom = TabHeight();
	Invalidate(aFrame);
}
//------------------------------------------------------------------------------
void	TScrollTabView::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case TBUTTON_SCRL_LEFT_CLICKED:
		TButtonScrlLeftClicked();
		break;
	case TBUTTON_SCRL_RIGHT_CLICKED:
		TButtonScrlRightClicked();
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
void	TScrollTabView::MouseDown(BPoint oPoint)
{
	SUPER::MouseDown(oPoint);
	TMouseDownManager::ManageMouseDown(oPoint);
	CheckSelectionChange();
}
//------------------------------------------------------------------------------
void	TScrollTabView::MouseDragBeginLeft(BPoint oBeginPoint, BPoint oPoint)
{
	mScrlPosOnDragBegin = mScrlPos;
}
//------------------------------------------------------------------------------
void	TScrollTabView::MouseDraggingLeft(BPoint oBeginPoint, BPoint oPoint)
{
	float	aMax;

	if ( !mShowScrlButtons && mScrlPos == 0.0 || CountTabs() == 0 )
		return;
	mScrlPos = mScrlPosOnDragBegin+oBeginPoint.x-oPoint.x;
	if ( mScrlPos < 0.0 ) {
		mScrlPos = 0.0;
	}
	aMax = -SUPER::TabFrame(static_cast<int32>(0)).left+SUPER::TabFrame(CountTabs()-1).left;
	if ( mScrlPos > aMax ) {
		mScrlPos = aMax;
	}
	InvalidateTabs();
}
//------------------------------------------------------------------------------
void	TScrollTabView::MouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg)
{
	SUPER::MouseMoved(oPoint, oTransit, oMsg);
	TMouseDownManager::ManageMouseMoved(oPoint, oTransit, oMsg);
}
//------------------------------------------------------------------------------
void	TScrollTabView::MouseUp(BPoint oPoint)
{
	SUPER::MouseUp(oPoint);
	TMouseDownManager::ManageMouseUp(oPoint);
}
//------------------------------------------------------------------------------
BTab*	TScrollTabView::RemoveTab(int32 oTabIdx)
{
	BTab*	aTab;

	aTab = SUPER::RemoveTab(oTabIdx);
	UpdateScrollButtons();
	CheckSelectionChange();
	return aTab;
}
//------------------------------------------------------------------------------
BRect	TScrollTabView::TabFrame(int32 oTabIdx) const
{
	float	aOffset = mScrlPos;
	BRect	aFrame = SUPER::TabFrame(oTabIdx);

	if ( mShowScrlButtons ) {
		aOffset -= mTButtonLeft->Frame().right;
	}
	aFrame.left -= aOffset;
	aFrame.right -= aOffset;
	return aFrame;
}
//------------------------------------------------------------------------------
void	TScrollTabView::TButtonScrlLeftClicked()
{
	int32	i;
	float	aMargin;

	if ( CountTabs() == 0 )
		return;
	aMargin = SUPER::TabFrame(static_cast<int32>(0)).left;
	i = 0;
	while ( i < CountTabs() && SUPER::TabFrame(i).left-aMargin < mScrlPos ) {
		i++;
	}
	i--;
	if ( i < 0 )
		return;
	mScrlPos = SUPER::TabFrame(i).left-aMargin;
	InvalidateTabs();
}
//------------------------------------------------------------------------------
void	TScrollTabView::TButtonScrlRightClicked()
{
	int32	i;
	float	aMargin;

	if ( CountTabs() == 0 )
		return;
	aMargin = SUPER::TabFrame(static_cast<int32>(0)).left;
	i = 0;
	while ( i < CountTabs() && SUPER::TabFrame(i).left-aMargin <= mScrlPos ) {
		i++;
	}
	if ( i >= CountTabs() )
		return;
	mScrlPos = SUPER::TabFrame(i).left-aMargin;
	InvalidateTabs();
}
//------------------------------------------------------------------------------
void	TScrollTabView::UpdateScrollButtons()
{
	float	aTotalTabWidth;

	if ( CountTabs() == 0 ) {
		aTotalTabWidth = 0.0;
	} else {
		// スクロールさせる前の位置から計算するので SUPER の TabFrame() をコール
		// タブの下の方のぴろんと出てる部分は TabFrame() に含まれないようなので、その分を加える。
		// 何ピクセルか直接はわからないが、TabFrame(0).left と同じと推測できる
		aTotalTabWidth = SUPER::TabFrame(static_cast<int32>(0)).left+SUPER::TabFrame(CountTabs()-1).right;
	}
	if ( aTotalTabWidth > Bounds().Width() ) {
		mShowScrlButtons = true;
		if ( mTButtonLeft->IsHidden() ) {
			mTButtonLeft->Show();
		}
		if ( mTButtonRight->IsHidden() ) {
			mTButtonRight->Show();
		}
	} else {
		mShowScrlButtons = false;
		mScrlPos = 0.0;
		if ( !mTButtonLeft->IsHidden() ) {
			mTButtonLeft->Hide();
		}
		if ( !mTButtonRight->IsHidden() ) {
			mTButtonRight->Hide();
		}
	}
}
//------------------------------------------------------------------------------
//==============================================================================
