/*==============================================================================
		文字列操作ユーティリティー関数群	for Zeta	Ver 1.11
		Copyright (C) 1999-2004 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/11/15 (Sat) 開発着手。
 1.00 | 2003/11/15 (Sat) add 時代の関数を元に構築。
 1.10 | 2004/08/18 (Wed) relative_path() を作った。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "StringUtils.h"
//------------------------------------------------------------------------------
// BeOS
// C++
#include <algorithm>
// Add
// Proj
//------------------------------------------------------------------------------
//==============================================================================
string	add_ext(const string& oOrgFName, const string& oExt)
{
	string::size_type	aPeriPos;	// ピリオド位置
	string::size_type	aSlashPos;	// フォルダ区切り位置
	string				aNewFName = oOrgFName;
	
	aPeriPos = aNewFName.rfind('.');
	aSlashPos = aNewFName.rfind('/');
	if ( static_cast<int32>(aPeriPos) <= static_cast<int32>(aSlashPos) ) {	// 有効なピリオドがない
		aNewFName += "."+oExt;
	}
	return aNewFName;
}
//------------------------------------------------------------------------------
string	reext(const string& oOrgFName, const string& oExt)
{
	string::size_type	aPeriPos;	// ピリオド位置
	string::size_type	aSlashPos;	// フォルダ区切り位置
	string				aNewFName = oOrgFName;
	
	aPeriPos = aNewFName.rfind('.');
	aSlashPos = aNewFName.rfind('/');
	if ( static_cast<int32>(aPeriPos) > static_cast<int32>(aSlashPos) ) {	// 有効なピリオドがある
		aNewFName.erase(aPeriPos+1);
	} else {
		aNewFName += ".";
	}
	aNewFName += oExt;
	return aNewFName;
}
//------------------------------------------------------------------------------
string	extract_file_body(const string& oFullPath)
{
	string::size_type	aExtractBeginPos;	// 抜き出し開始位置
	string::size_type	aExtractEndPos;		// 抜き出し終了位置

	aExtractBeginPos = oFullPath.rfind('/')+1;
	aExtractEndPos = oFullPath.rfind('.');
	if ( aExtractEndPos < aExtractBeginPos ) {
		aExtractEndPos = oFullPath.length();
	}
	return oFullPath.substr(aExtractBeginPos, aExtractEndPos-aExtractBeginPos);
}
//------------------------------------------------------------------------------
string	extract_file_ext(const string& oFullPath)
{
	string::size_type	aDemiPos;	// '.'の位置
	string				aExt;		// 抜き出した拡張子（返値）

	aDemiPos = oFullPath.rfind('.');
	if ( aDemiPos == string::npos ) {
		// 拡張子無し
	} else {
		aExt = oFullPath.substr(aDemiPos+1);
	}
	return aExt;
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
共通使用関数
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
string	extract_file_folder_path_kernel(const string& oFullPath, int oAddOffset)
{
	string::size_type	aDemiPos;	// '/' の位置
	string				aPath;		// 抜き出したパス（返値）

	aDemiPos = oFullPath.rfind('/');
	if ( aDemiPos == string::npos ) {
		//パス無し
	} else {
		aPath = oFullPath.substr(0, aDemiPos+oAddOffset);
	}
	return aPath;
}
//------------------------------------------------------------------------------
string	extract_file_name(const string& oFullPath)
{
	string::size_type	aDemiPos;	// '/'の位置
	string				aName;		// 抜き出したファイル名（返値）

	aDemiPos = oFullPath.rfind('/');
	if ( aDemiPos == string::npos ) {	// パス無し
		aName = oFullPath;
	} else {
		aName = oFullPath.substr(aDemiPos+1);
	}
	return aName;
}
//------------------------------------------------------------------------------
string	relative_path(const string& oFrom, const string& oTo)
{
	int32				aBackNum;
	pair<string::const_iterator, string::const_iterator>	aDiffPos;
	string				aPath;

	aDiffPos = mismatch(oFrom.begin(), oFrom.end(), oTo.begin());
	aBackNum = count(aDiffPos.first, oFrom.end(), '/');
	if ( aBackNum == 0 ) {
		aPath = "./";
	}
	for ( int32 i = 0 ; i < aBackNum ; i++ ) {
		aPath += "../";
	}
	while ( aDiffPos.second != oTo.begin() && *(aDiffPos.second-1) != '/' ) {
		aDiffPos.second--;
	}
	aPath += oTo.substr(aDiffPos.second-oTo.begin(), oTo.end()-aDiffPos.second);
	return aPath;
}
//------------------------------------------------------------------------------
string	trim(const string& oSrc)
{
	string::size_type	aBeginPos;
	string::size_type	aEndPos;

	if ( oSrc.empty() )
		return "";
	// 先頭
	aBeginPos = oSrc.find_first_not_of(" \t");
	if ( aBeginPos == string::npos )
		return "";
	// 末尾
	aEndPos = oSrc.find_last_not_of(" \t");
	return oSrc.substr(aBeginPos, aEndPos-aBeginPos+1);
}
//------------------------------------------------------------------------------
//==============================================================================
