#ifndef	___ToolBar_H_
#define	___ToolBar_H_
//------------------------------------------------------------------------------
// BeOS
#include <Bitmap.h>
#include <Control.h>
#include <Message.h>
#include <PictureButton.h>	// B_ONE_STATE_BUTTON など
#include <View.h>
// C++
#include <vector>
// Add2
#include "ToolTip.h"
//------------------------------------------------------------------------------
using namespace	std;
//------------------------------------------------------------------------------
enum	{ TB_STATE_ENABLED_OFF, TB_STATE_ENABLED_ON, TB_STATE_DISABLED_OFF, TB_STATE_DISABLED_ON,
		TB_STATE_HOT_OFF, TB_STATE_HOT_ON /* 実際には使われない */, TB_STATE_END };
//------------------------------------------------------------------------------
class	TToolButton;
//==============================================================================
class	TToolBar : public BView
{
public:
	TToolBar(BRect oFrame, const char* oName, uint32 oResizingMode = B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
	~TToolBar();

	// アイテムの追加（所有権移動）と削除
	bool				AddItem(TToolButton* oItem);
	bool				AddItem(TToolButton* oItem, int32 oIndex);
	bool				AddSeparatorItem();
	TToolButton*		RemoveItem(int32 oIndex);
	bool				RemoveItem(TToolButton* oItem);

	// アイテムへのアクセス
	int32				IndexOf(TToolButton* oItem) const;
	int32				CountItems() const;
	TToolButton*		ItemAt(int32 oIndex) const;
	TToolButton*		FindItem(const char* oName) const;
	TToolButton*		FindPushed(int32 oBegin = 0) const;

	// 見た目
	status_t			SetFlat(bool oFlat);
	bool				IsFlat() const;

	// サイズ
	virtual void		GetPreferredSize(float* oWidth, float* oHeight);

	// 有効状態
	virtual void		SetEnabled(bool oEnabled);
	bool				IsEnabled() const;

	// 各種設定
	void				RelocateItems(int32 oIndex = 0);	// oIndex 以降のアイテムを再配置

	virtual status_t	SetTargetForItems(BHandler* oHandler);
	virtual status_t	SetTargetForItems(BMessenger oMsger);

private:
	typedef	BView	SUPER;

	friend class	TToolButton;

	virtual void		AttachedToWindow();
	virtual void		Draw(BRect oUpdateRect);

	bool					mEnabled;
	bool					mFlat;				// クールバータイプ
	float					mMaxItemHeight;		// ツールボタン中の最高の高さ
	vector<TToolButton*>	mItems;				// ツールボタンへのポインタ群
};
//==============================================================================
class	TToolButton : public BControl
{
public:
	// oEnabledOff は NULL 可だが、アタッチされるまでには指定されていなくてはならない
	// oLabel は NULL 不可（後に落ちる）
	TToolButton(BRect oFrame, const char* oName, BBitmap* oEnabledOff, BMessage* oMsg,
			uint32 oBehavior = B_ONE_STATE_BUTTON);
	TToolButton(BRect oFrame, const char* oName, BBitmap* oEnabledOff, const char* oLabel, BMessage* oMsg,
			uint32 oBehavior = B_ONE_STATE_BUTTON);
	~TToolButton();

	// イメージ
	status_t			SetImage(int32 oWhich, BBitmap* oImg);
	status_t			SetDefaultImages();						// 設定されていないビットマップを作成 | 1 つ以上作成した
	status_t			ClearImages();							// ビットマップを全てクリア（ TB_STATE_ENABLED_OFF は除く）

	// ラベル
	virtual void		SetLabel(const char* oLabel);

	// 配置
	virtual void		SetAlignment(alignment oForImage, alignment oForLabel);	// 未実装
	void				GetAlignment(alignment* oForImage, alignment* oForLabel) const;	// 未実装
	virtual void		SetDivider(float oDivider);				// 画像を設定した時は自動設定
	virtual void		ResizeToPreferred();

	// ツールチップ
	status_t			SetTip(const string& oTipStr);
	string				Tip() const;

	// ONE / TWO STATE
	virtual status_t	SetBehavior(uint32 oBehavior);
	uint32				Behavior() const;

	// ラジオモード
	virtual status_t	SetRadioMode(bool oFlag);				// 兄弟全員のラジオモードを設定
	bool				IsRadioMode() const;

	virtual void		SetValue(int32 oValue);

private:
	typedef	BControl	SUPER;

	friend class	TToolBar;

	void			AdjustDivider();
	virtual void	AttachedToWindow();
	void			CalcDrawPoints();
	void			ConvertFilledCopy(const BBitmap* oSrc, BBitmap** oDest, const rgb_color& oDestColor);		// 色を変換しながらコピー
	void			ConvertTransparentCopy(const BBitmap* oSrc, BBitmap** oDest, const rgb_color& oDestColor);	// 色を変換しながらコピー
	virtual void	Draw(BRect oUpdateRect);
	void			GetBrothers(vector<TToolButton*>* oBrothers);	// 兄弟の取得（自分は含まない）
	void			Init(const char* oName, BBitmap* oEnabledOff, uint32 oBehavior);
	float			MinWidth() const;
	virtual void	MouseDown(BPoint oPoint);
	virtual void	MouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg);
	virtual void	MouseUp(BPoint oPoint);
	virtual void	WindowActivated(bool oActive);

	bool			mChanging;					// OFF → ON or ON → OFF の途中
	bool			mHot;						// ホット
	bool			mRadioMode;					// ラジオモードかどうか
	uint32			mBehavior;					// ONE / TWO STATE
	float			mDivider;
	alignment		mImageAlign;				// 画像の配置
	alignment		mLabelAlign;				// ラベルの配置
	BBitmap*		mImages[TB_STATE_END];		// 表示する画像
	BPoint			mImagePoint;				// 画像を表示する位置
	BPoint			mLabelPoint;				// ラベルを表示する位置
	TToolBar*		mOwner;						// TToolBar によって設定される
	TToolTip*		mToolTip;					// ツールチップ
};
//==============================================================================
class	TSeparatorButton : public TToolButton
{
public:
	TSeparatorButton(float oHeight);
	~TSeparatorButton();

private:
	typedef	TToolButton	SUPER;

	virtual void	AttachedToWindow();
	virtual void	Draw(BRect oUpdateRect);
};
//==============================================================================
#endif	// ___ToolBar_H_
