/*==============================================================================
		ツールチップコンポーネント		for ZETA		Ver 1.03
		Copyright (C) 2001-2006 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/12/07 (Sun) 開発着手。
 1.00 | 2003/12/07 (Sun) add 時代のクラスを元に構築。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "ToolTip.h"
//------------------------------------------------------------------------------
// BeOS
#include <Application.h>
#include <Messenger.h>
#include <interface/StringView.h>
// C++
#include <strstream>
// Add2
#include "FUConstants.h"
#include "MiscDebug.h"
// Proj
//------------------------------------------------------------------------------
bigtime_t	TToolTip::smLastHideTime = 0;
//------------------------------------------------------------------------------
const uint32	HIDE_TOOL_TIP_WINDOW_RQTD = 'TThw';
const uint32	MOUSE_ENTERED = 'TTme';
const uint32	MOUSE_EXITED = 'TTmx';
const uint32	OPEN_TOOL_TIP_WINDOW_RQTD = 'TTow';
//------------------------------------------------------------------------------
const char		WIN_NAME_TOOL_TIP_PRFX[] = "TlTipWin_";
const bigtime_t	CONTINUOUS_SHOW_INTERVAL = 200*1000;	// 隣のツールボタンのチップを続けざまに表示してよい間隔（セパレータを通過することを考慮して多少長めに）
const float		TOOL_TIP_WIN_OFFSET_X = -16.0;
const float		TOOL_TIP_WIN_OFFSET_Y = 16.0;
//==============================================================================
TToolTip::TToolTip(const char* oName)
		: SUPER(oName)
{
	// メンバ初期化
	mDelayTime = 1*1000*1000;
	mShowTime = 5*1000*1000;
	mDelayNotifyer = NULL;
	mTgtView = NULL;
	mSpy = new TTTSpy(this);
}
//------------------------------------------------------------------------------
TToolTip::~TToolTip()
{
	if ( mTgtView == NULL ) {
		delete mSpy;
	}
}
//------------------------------------------------------------------------------
bigtime_t	TToolTip::DelayTime() const
{
	return mDelayTime;
}
//------------------------------------------------------------------------------
BWindow*	TToolTip::FindToolTipWindow()
{
	int32		i = 0;
	BWindow*	aWin;

	while ( (aWin = be_app->WindowAt(i)) != NULL ) {
		if ( string(aWin->Name()) == ToolTipWindowName() )
			break;
		i++;
	}
	return aWin;
}
//------------------------------------------------------------------------------
void	TToolTip::Hide()
{
	BMessage	aQuitMsg(B_QUIT_REQUESTED);
	BWindow*	aToolTipWin;

	if ( (aToolTipWin = FindToolTipWindow()) != NULL ) {
		BMessenger(aToolTipWin).SendMessage(&aQuitMsg);
		smLastHideTime = system_time();
//		DBEXP("TToolTip::Hide()", ToolTipWindowName());
	} else {
//		DBEXP("TToolTip::Hide()", "find failed");
	}
}
//------------------------------------------------------------------------------
void	TToolTip::HideToolTipWindowRqtd()
{
	Hide();
}
//------------------------------------------------------------------------------
bool	TToolTip::IsAnyToolTipWindowOpened() const
{
	int32		i = 0;
	BWindow*	aWin;

	while ( (aWin = be_app->WindowAt(i)) != NULL ) {
		if ( string(aWin->Name()).find(WIN_NAME_TOOL_TIP_PRFX) != string::npos )
			break;
		i++;
	}
	return (aWin != NULL);
}
//------------------------------------------------------------------------------
void	TToolTip::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case HIDE_TOOL_TIP_WINDOW_RQTD:
		HideToolTipWindowRqtd();
		break;
	case MOUSE_ENTERED:
		MouseEntered();
		break;
	case MOUSE_EXITED:
		MouseExited();
		break;
	case OPEN_TOOL_TIP_WINDOW_RQTD:
		OpenToolTipWindowRqtd();
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
void	TToolTip::MouseEntered()
{
	BMessage	aDelayMsg(OPEN_TOOL_TIP_WINDOW_RQTD);

//	DBEXP("TToolTip::MouseEntered","");
	// ディレイ通知者を作成
	if ( mDelayNotifyer == NULL ) {
		if ( IsAnyToolTipWindowOpened() || system_time()-smLastHideTime < CONTINUOUS_SHOW_INTERVAL ) {
			BMessenger(this).SendMessage(&aDelayMsg);
		} else {
			mDelayNotifyer = new BMessageRunner(BMessenger(this),&aDelayMsg,mDelayTime,1);
		}
	}
}
//------------------------------------------------------------------------------
void	TToolTip::MouseExited()
{
	BMessage	aSendMsg(HIDE_TOOL_TIP_WINDOW_RQTD);

//	DBEXP("TToolTip::MouseExited","");
	// ディレイ通知者を途中破棄
	if ( mDelayNotifyer != NULL ) {
		delete mDelayNotifyer;
		mDelayNotifyer = NULL;
	}
	// ツールチップウィンドウを閉じる（メッセージキューに OPEN_TOOL_TIP_WINDOW_RQTD が溜まっているかもしれないので直接 Hide() は呼ばない）
	BMessenger(this).SendMessage(&aSendMsg);
}
//------------------------------------------------------------------------------
void	TToolTip::OpenToolTipWindowRqtd()
{
	uint32			aButtons;
	BPoint			aPoint;
	TToolTipWindow*	aToolTipWin;

	// ディレイ通知者を破棄
	delete mDelayNotifyer;
	mDelayNotifyer = NULL;
	// ツールチップウィンドウを表示
//	DBEXP("TToolTip::OpenToolTipWindowRqtd - already shown?",FindToolTipWindow() != NULL);
	if ( /*FindToolTipWindow() == NULL && 隣のチップを連続表示するためにはこれがあるとダメ */ mTipStr != "" && Target()->Window()->IsActive() ) {
		mTgtView->GetMouse(&aPoint,&aButtons);
		// マウス押下時は表示しない（ツールバーで、マウス押下状態でマウスをずらすとものすごい勢いでツールチップが更新されるのを回避するため）
		if ( aButtons == 0 ) {
			mTgtView->ConvertToScreen(&aPoint);
			aPoint += BPoint(TOOL_TIP_WIN_OFFSET_X,TOOL_TIP_WIN_OFFSET_Y);
			aToolTipWin = new TToolTipWindow(BRect(aPoint,aPoint),ToolTipWindowName(),mTipStr,mShowTime);
			aToolTipWin->Show();
		}
	}
}
//------------------------------------------------------------------------------
bigtime_t	TToolTip::ShowTime() const
{
	return mShowTime;
}
//------------------------------------------------------------------------------
status_t	TToolTip::SetDelayTime(bigtime_t oDelayTime)
{
	if ( oDelayTime < 0 )
		return B_ERROR;
	mDelayTime = oDelayTime;
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TToolTip::SetShowTime(bigtime_t oShowTime)
{
	if ( oShowTime < 0 )
		return B_ERROR;
	mShowTime = oShowTime;
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TToolTip::SetTarget(BView* oTgt)
{
	BWindow*	aTgtParent;

	aTgtParent = oTgt->Window();
	if ( aTgtParent == NULL )
		return B_ERROR;
	// ターゲットの解除
	if ( mTgtView != NULL && mTgtView->Window() != NULL ) {
		mTgtView->Window()->RemoveHandler(this);
	}
	// ターゲットの設定
	mTgtView = oTgt;
	mSpy->SetTarget(mTgtView);
	aTgtParent->AddHandler(this);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TToolTip::SetTip(const string& oTipStr)
{
	mTipStr = oTipStr;
	return B_OK;
}
//------------------------------------------------------------------------------
BView*		TToolTip::Target() const
{
	return mTgtView;
}
//------------------------------------------------------------------------------
string		TToolTip::Tip() const
{
	return mTipStr;
}
//------------------------------------------------------------------------------
string	TToolTip::ToolTipWindowName() const
{
	if ( Name() == NULL )
		return string(WIN_NAME_TOOL_TIP_PRFX);
	else
		return string(WIN_NAME_TOOL_TIP_PRFX)+Name();
}
//------------------------------------------------------------------------------
//==============================================================================
TToolTipWindow::TToolTipWindow(BRect oFrame,const string& oName,const string& oTipStr,bigtime_t oShowTime)
		: SUPER(oFrame,oName.c_str(),B_BORDERED_WINDOW_LOOK,B_FLOATING_APP_WINDOW_FEEL,B_AVOID_FRONT|B_AVOID_FOCUS)
{
	BMessage		aQuitMsg(B_QUIT_REQUESTED);
	BStringView*	aStrView;

	// 表示
	aStrView = new BStringView(oFrame.OffsetToCopy(B_ORIGIN),oTipStr.c_str(),(" "+oTipStr+" ").c_str());
	aStrView->SetViewColor(FUC_COLOR_TOOL_TIP_WIN_BG);
	aStrView->ResizeToPreferred();
	ResizeTo(aStrView->Bounds().right,aStrView->Bounds().bottom);
	AddChild(aStrView);
	// タイマー
//	DBEXP("TToolTipWindow::TToolTipWindow",oShowTime);
	mRunner = new BMessageRunner(BMessenger(this),&aQuitMsg,oShowTime,1);
}
//------------------------------------------------------------------------------
TToolTipWindow::~TToolTipWindow()
{
	delete mRunner;
}
//------------------------------------------------------------------------------
//==============================================================================
TTTSpy::TTTSpy(BHandler* oTransferTgt)
		: SUPER(B_MOUSE_MOVED)
{
	mAlreadyShown = false;
	mPrevMouseTgt = NULL;
	mTgt = NULL;
	mTransferTgt = oTransferTgt;
}
//------------------------------------------------------------------------------
TTTSpy::~TTTSpy()
{
}
//------------------------------------------------------------------------------
filter_result	TTTSpy::Filter(BMessage* oMsg,BHandler** oMouseTgt)
{
	BLooper*	aTransferLooper;
	BMessage	aSendMsg;

	aTransferLooper = mTransferTgt->Looper();
	if ( aTransferLooper != NULL ) {
		if ( *oMouseTgt == mTgt && mPrevMouseTgt != mTgt && !mAlreadyShown ) {	// ENTER
//			DBEXPIF(string(mTgt->Name()) == "aaa","TTTSpy::Filter - ENTER","");
			aSendMsg.what = MOUSE_ENTERED;
			BMessenger(mTransferTgt).SendMessage(&aSendMsg);
			mAlreadyShown = true;
		} else if ( *oMouseTgt != mTgt && mPrevMouseTgt == mTgt ) {	// EXIT
//			DBEXPIF(string(mTgt->Name()) == "aaa","TTTSpy::Filter - EXIT","");
			aSendMsg.what = MOUSE_EXITED;
			BMessenger(mTransferTgt).SendMessage(&aSendMsg);
			mAlreadyShown = false;
		}
		mPrevMouseTgt = *oMouseTgt;
	}
	return B_DISPATCH_MESSAGE;
}
//------------------------------------------------------------------------------
status_t	TTTSpy::SetTarget(BHandler* oTgt)
{
	BLooper*	aNewLooper = oTgt->Looper();

	if ( aNewLooper == NULL )
		return B_ERROR;
	// 解除
	if ( mTgt != NULL && mTgt->Looper() != NULL ) {
		mTgt->Looper()->RemoveCommonFilter(this);
	}
	// 設定
	aNewLooper->AddCommonFilter(this);
	mTgt = oTgt;
	return B_OK;
}
//------------------------------------------------------------------------------
//==============================================================================
