#ifndef	___ToolTip_H_
#define	___ToolTip_H_
//------------------------------------------------------------------------------
// BeOS
#include <Handler.h>
#include <MessageFilter.h>
#include <MessageRunner.h>
#include <View.h>
#include <Window.h>
// C++
#include <string>
//------------------------------------------------------------------------------
using namespace	std;
//------------------------------------------------------------------------------
class	TTTSpy;
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
ターゲットに TTTSpy が張り付き、マウスの出入りを監視。
TTTSpy から TToolTip にマウスの情報が送られてくる。
TToolTip が TToolTipWindow の表示・非表示を制御。
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TToolTip : public BHandler
{
public:
	TToolTip(const char* oName = NULL);
	~TToolTip();

	// 表示対象
	status_t	SetTarget(BView* oTgt);	// oTgt はアタッチされていなくてはならない
	BView*		Target() const;

	// 文字列
	status_t	SetTip(const string& oTipStr);
	string		Tip() const;

	// 表示までの待機時間
	status_t	SetDelayTime(bigtime_t oDelayTime);
	bigtime_t	DelayTime() const;

	// 表示時間
	status_t	SetShowTime(bigtime_t oShowTime);
	bigtime_t	ShowTime() const;

	// 強制表示中止
	void		Hide();

private:
	typedef	BHandler	SUPER;

	BWindow*		FindToolTipWindow();				// ウィンドウを見つける
	void			HideToolTipWindowRqtd();
	bool			IsAnyToolTipWindowOpened() const;	// 誰かのツールチップウィンドウが表示されている
	virtual void	MessageReceived(BMessage* oMsg);
	void			MouseEntered();
	void			MouseExited();
	void			OpenToolTipWindowRqtd();
	string			ToolTipWindowName() const;			// ウィンドウの名前（自分のツールチップウィンドウの）

	static bigtime_t	smLastHideTime;					// 最後にツールチップウィンドウを閉じた時刻

	bigtime_t		mDelayTime;
	bigtime_t		mShowTime;
	string			mTipStr;
	BMessageRunner*	mDelayNotifyer;
	BView*			mTgtView;
	TTTSpy*			mSpy;
};
//==============================================================================
class	TTTSpy : public BMessageFilter
{
public:
	TTTSpy(BHandler* oTransferTgt);
	~TTTSpy();

	status_t	SetTarget(BHandler* oTgt);

private:
	typedef	BMessageFilter	SUPER;

	virtual filter_result	Filter(BMessage* oMsg, BHandler** oMouseTgt);

	bool		mAlreadyShown;	// 既にツールチップを表示した
	BHandler*	mPrevMouseTgt;
	BHandler*	mTgt;			// マウスオーバーを監視する BView
	BHandler*	mTransferTgt;	// マウスオーバー検出の伝達先

};
//==============================================================================
class	TToolTipWindow : public BWindow
{
public:
	TToolTipWindow(BRect oFrame, const string& oName, const string& oTipStr, bigtime_t oShowTime);
	~TToolTipWindow();

private:
	typedef	BWindow	SUPER;

	BMessageRunner*	mRunner;
};
//==============================================================================
#endif	// ___ToolTip_H_
